/*********************************************************************
LCD8_port_P0.c       (sterowanie wywietlaczem poprzez 8-linii danych)

Program demonstruje sposb obsugi wywietlacza LCD podczonego
do portu P0 poprzez 8-linii danych.
*********************************************************************/
#include "51xD2.h"             // zbir definiujcy rejestry procesora 

void LCD_ctrlWR ( BYTE cmde);
void LCD_dataWR ( BYTE byte);
void LCD_ini (void);
BYTE check_busy (void);

#define WAIT  for(var=0; var<0x2000;var++)       // (1) !
//#define WAIT  while (check_busy() )            // (2) !

//-------------------------------------------------------------------
void main(void) { 
  BYTE i;
  LCD_ini();             
  for(i=0;i<10;i++) {
       LCD_dataWR('P'+i);
  }
  while(1); 
}

/*===================================================================
 Wywietlacz LCD podczony jest do portu P0 i sterowany programowo:
   LCD          uP
   -------------------
   DB0        = P0_0
   DB1        = P0_1
   DB2        = P0_2
   DB3        = P0_3
   DB4        = P0_4
   DB5        = P0_5
   DB6        = P0_6
   DB7        = P0_7

   D/C~ (RS)  = P2_6        Data/Control~
   R/W~       = P2_7        Read/Write~
   E          = P3_6        Enable sterowane przez bramk 74HC00
                            Linia P3_7 stale jest rna 1 po reset ! 
===================================================================*/
#define DATA_LCD P0             // wywietlacz podczony do portu P0  
#define DC_LCD   P2_6                   // linia D/C~ (Data/Control~)
#define RW_LCD   P2_7                     // linia R/W~ (Read/Write~) 
#define E_LCD    P3_6                             // linia E (Enable) 

//--------------------------------- zapis 8-bitw do wywietlacza LCD 
void write_byte ( BYTE val) {
  E_LCD = 1;                                   // Enable=1 nieaktywna 
  RW_LCD = 0;                                  // linia zapisu R/W~=0  
  DATA_LCD = val;                     // wystawienie danych do zapisu  

  E_LCD = 0;               // linia Enable E=0: zapis do wywietlacza  
  E_LCD = 1;                       // linia Enable E=0: koniec zapisu  
  DATA_LCD = 0xFF;           // ustawienie linii danych w stan wysoki
}

//--------------- zapis sowa sterujcego Control do wywietlacza LCD 
void LCD_ctrlWR ( BYTE cmde) {
    WORD var;
    WAIT;                                                 // poczekaj 
    DC_LCD = 0;                        // wybierz rejestr Control LCD   
    write_byte(cmde);            // zapis bajtu polecenia sterujcego
}

//----------------------------- zapis danych Data do wywietlacza LCD 
void LCD_dataWR ( BYTE byte) {
    WORD var;
    WAIT;                                                 // poczekaj 
    DC_LCD = 1;                    // wybierz rejestr danych Data LCD  
    write_byte(byte);                          // zapis  bajtu danych
}

// --------------------- sprawdzenie flagi zajtoci wywietlacza LCD 
BYTE check_busy (void) {
    BYTE bf;
    DC_LCD = 0;       // linia D/C~=0 : odczyt sowa Control (status)  
    RW_LCD = 1;                              // linia R/W~=1 : odczyt  
    E_LCD = 1; 
    E_LCD = 0; 
    bf = DATA_LCD & 0x80; // najstarszy bit informuje o zajtoci LCD 
    E_LCD = 1; 
    return (bf);
}

//------------------------------------ inicjalizacja wywietlacza LCD
void LCD_ini (void) {
BYTE init_tab[] = {0x01,0x0F};                   // LCD_RESET, LCD_ON
WORD i,j;
  for(j=0;j<0x1000;j++);      // poczekaj na zerowanie w LCD po reset
  for (i = 0; i < sizeof init_tab; i++) { 
       LCD_ctrlWR(init_tab[i]);          // zapis danych inicjujcych
 }
}

