/********************************************************************
LCD4_port_P03-P00.c       (Wywietlacz sterowany przez 4 linie danych
                               - modsza cze portu P0 zawiera dane)

Program demonstruje sposb obsugi wywietlacza LCD podczonego do
portu poprzez 4-linie danych, do mdszej czci portu P0.
********************************************************************/
#include "51xD2.h"            // zbir definiujcy rejestry procesora

void LCD_ctrlWR ( BYTE cmde);
void LCD_dataWR ( BYTE byte);
void LCD_ini (void);
BYTE check_busy (void);

#define WAIT  for(var=0; var<0x2000;var++)   // (1) !
//#define WAIT  while (check_busy() )            // (2) !

//-------------------------------------------------------------------
void main(void) {
  BYTE i;
  LCD_ini();

  for(i=0;i<16;i++) {
       LCD_dataWR('@'+i);
  }

  while(1);
}

/*====================================================================
Wywietlacz LCD podczony jest do portu P0:
   LCD          uP
   -------------------
   DB4        = P0_4
   DB5        = P0_5
   DB6        = P0_6
   DB7        = P0_7

   D/C~ (RS)  = P0_4     Data/Control~
   R/W~       = P0_5     Read/Write~
   E          = P0_6     Enable
====================================================================*/

#define DATA_LCD P0              // wywietlacz podczony do portu P0
#define DC_LCD   P0_4                     // linia D/C~ (Data/Control)
#define RW_LCD   P0_5                       // linia R/W~ (Read/Write)
#define E_LCD    P0_6                              // linia E (Enable)

//---------------------------------- zapis 4-bitw do wywietlacza LCD
void write_nibble ( BYTE nibble) {
  BYTE x;
  E_LCD = 0;                        // Enable=1 wywietlacz nieaktywny
  RW_LCD = 0;                                   // linia zapisu R/W~=0
  x= DATA_LCD & 0xF0;
  nibble &= 0x0f;                // tylko 4-modsze bity bd zapisane
  DATA_LCD = nibble | x;    //dane s przekazywane na liniach P0_3...P0_0
  E_LCD = 1;                // linia Enable E=0: zapis do wywietlacza
  E_LCD = 0;                        // linia Enable E=1: koniec zapisu
  DATA_LCD = x | 0x0F;                // powrt linii danych do stanu 0xFF
}

//---------------- zapis sowa sterujcego Control do wywietlacza LCD
void LCD_ctrlWR ( BYTE cmde) {
    WORD var;
    WAIT;                                                  // poczekaj
    DC_LCD = 0;                         // wybierz rejestr Control LCD
    write_nibble(cmde >> 4);   // zapis starszej czwrki bajtu Control
    write_nibble(cmde);        // zapis modszej czwrki bajtu Control
}

//------------------------------ zapis danych Data do wywietlacza LCD
void LCD_dataWR ( BYTE byte) {
    WORD var;
    WAIT;                                                  // poczekaj
    DC_LCD = 1;                     // wybierz rejestr danych Data LCD
    write_nibble(byte >> 4);      // zapis starszej czwrki bajtu Data
    write_nibble(byte);           // zapis modszej czwrki bajtu Data
}

//------------------------------------- inicjalizacja wywietlacza LCD
void LCD_ini (void) {
BYTE init_tab[] = {0x33,0x32,0x28,0x01,0x0F}; //polecana inicjalizacja
WORD i,j;
  DATA_LCD =0x0F;                       // Enable=1 wywietlacz nieaktywny
  for(j=0;j<0x1000;j++);
  for (i = 0; i < sizeof init_tab; i++) {
       LCD_ctrlWR(init_tab[i]);           // zapis danych inicjujcych
 }
}

// ---------------------- sprawdzenie flagi zajtoci wywietlacza LCD
BYTE check_busy (void) {
    BYTE bf;
    DC_LCD = 0;       // linia D/C~=0 : odczyt sowa Control (statusu)
    RW_LCD = 1;                               // linia R/W~=1 : odczyt
    E_LCD = 1;
    bf = DATA_LCD & 0x08;  // najstarszy bit w czwrce informuje o zajtoci LCD
    E_LCD = 0;
    E_LCD = 1;              // linia Enable E=0: odczyt z wywietlacza
    E_LCD = 0;                      // linia Enable E=1:koniec odczytu
    return (bf);
}


