/*****************************************************************
melody.c                                     (generowanie melodii)

Program uywa ukadu Timer2 do generowanie melodii. Melodia
przygotowana jest w postaci zapisu nutowego: wysoko tonu
i czas trwania tonu. Do odmierzania czasu trwania nuty uywany
jest Timer0. 
Generowany przebieg jest dostpny na wyjciu T2 (P1_0).
Czstotliwo generowana przez ukadu Timer2 dana jest wzorem:

   F_clock_out=Fosc/(4* (65536-RCAP2H,RCAP2L))

Uwaga: do linii P1_0 naley doczy suchawki o opornoci 
       wikszej od 300 Ohm.
*****************************************************************/
#include "51xD2.h"        // zbir definiujcy rejestry procesora

#define Fosc   24000000L              // czstotliwo oscylatora

// Czstotilowci nut w Hz
#define FreqC   523
#define FreqD   587
#define FreqE   659
#define FreqF   698
#define FreqG   784
#define FreqA   880
#define FreqB   988

// zawarto rejestrw RCAP2H,RCAP2L potrzebna do generowania 
// czstotliwoci: 
//            -TICKS_T2= 65536- RCAP2H,RCAP2L= ( Fosc/(4* freq))
#define TICKS_T2(freq)      ( - Fosc/(4* freq) )

#define NoteC TICKS_T2( FreqC )
#define NoteD TICKS_T2( FreqD )
#define NoteE TICKS_T2( FreqE )
#define NoteF TICKS_T2( FreqF )
#define NoteG TICKS_T2( FreqG )
#define NoteA TICKS_T2( FreqA )
#define NoteB TICKS_T2( FreqB )

// nutowy zapis melodii.
// czas trwania nuty wyraraony jest w "semkach" (1/8)
code WORD    Notes[][2] = {       
        {NoteE,2}, { NoteE,2}, { NoteF,2}, { NoteG,2},
        {NoteG,2}, { NoteF,2}, { NoteE,2}, { NoteD,2},
        {NoteC,2}, { NoteC,2}, { NoteD,2}, { NoteE,2},
        {NoteE,3}, { NoteD,1}, { NoteD,4},   
        {NoteE,2}, { NoteE,2}, { NoteF,2}, { NoteG,2},
        {NoteG,2}, { NoteF,2}, { NoteE,2}, { NoteD,2},
        {NoteC,2}, { NoteC,2}, { NoteD,2}, { NoteE,2},
        {NoteD,3}, { NoteC,1}, { NoteC,4},
};

// przygotowanie ukadu Timer2 do generowania czstotliwoci ------
void T2_init(void){
     T2MOD =0x02;         // tryb ukadu Timer2:  P1_0 clock_output
     T2CON =0x04;         // ...
}
#define one_msek  (- Fosc/(12 * 1000) )            // 1 milisekunda       
#define TL0RELOAD    one_msek     
#define TH0RELOAD  ( one_msek )>>8        
#define SECOND     1000     // 1 sek wymaga 1000 przerwa od Timer0
WORD timerM;                     

// przygotowanie ukadu Timer0 ------------------------------------
void T0_init(void) {
    TH0  = TH0RELOAD;               // zaadowanie czasu odliczania
    TL0  = TL0RELOAD;
    TMOD = TMOD | 0x01;                     // tryb 1 ukadu Timer0
    TR0  = 1;                                      //  Timer0 start
    ET0  = 1;                    // odblokowanie przerwa od Timer0
}

// obsuga przerwania ukadu Timer0 -------------------------------
void T0_int(void)  interrupt 1 using 1 { 
  TH0 = TH0RELOAD;                  // zaadowanie czasu odliczania
  TL0 = TL0RELOAD; 
  
   if (timerM>0) timerM--;   // odlicznia zadanej liczby milisekund
}

// czekanie zadan liczb milisekund ------------------------------
void delayM(WORD val) {          
  timerM = val;       
  while(timerM) {};           
}


// podprogram opnienia ------------------------------------------
void Delay (BYTE wait ) {
  TR2=1;
  delayM(wait*200);                        // 1/8 nuty trwa 200msek
  TR2=0;
  delayM(10);                                       // 10msek ciszy
}

void main(void) {
 BYTE i;
   T2_init();                        // przygotowanie ukadu Timer2
   T0_init();                        // przygotowanie ukadu Timer0
   EA  = 1;
   while (1) {                               // odtwarzanie melodii
    RCAP2L= Notes[i][0] ;                  // generowanie tonu nuty
    RCAP2H= Notes[i][0]>>8;
    Delay ( Notes[i][1] );                     // czas trwania nuty
    i=  (i+1) % ( (sizeof Notes )/ 4);      // kolejna nuta melodii
   }
 }
