/*****************************************************************
timers.c     (uycie wielu ukadw odmierzania sekund i milisekund)

Program pokazue jak uywa jednoczenie wielu licznikw (timers)
odliczjcych czas. Liczniki odliczj czas mierzony w 
milisekundach lub sekundach. Podstaw tego progranu jest 
podprogram obsugi przerwania ukadu Timer0.
Rezultatu niezalenych licznikw to:
- mrugajca dioda LED z okresem 200msek
- co 1sek pojawi si cyra od 0 do 9
- co 10sek zmieni si pozycja wywietlanych cyfr na wywietlaczu

 Uwaga:  Zdj zwor JP7 (Zezwolenie na dziaanie wywietlacza
         jest wymuszone opornikiem R22).        
         Doczy zewntrzn diod LED do JP5. Zwr uwag na 
         polaryzacj diody (zamie wyprowadzenia diody).
         Zaay zwor JP10 aby buzzer by syszalny.
*****************************************************************/
#include "51xD2.h"         // zbir definiujcy rejestry procesora 
#define  Buzzer     P1_3           // definicja adresu bitu Buzzer   
#define  LEDS       P2            // zesp diod wywietlacza LEDS
#define  LED        P1_2                   // zewntrzna dioda LED 

#define MaxTimerM   3  // liczba timerw odliczajcych milisekundy  
#define MaxTimerS   3      // liczba timerw odliczajcych sekundy      
         
#define TH0RELOAD  0xfc          //  0xfc18 = -1000 dla Fosc=12MHz
#define TL0RELOAD  0x18          //  ...    
#define SECOND     1000    // 1 sek wymaga 1000 przerwa od Timer0
idata BYTE timerM[MaxTimerM];              // zliczanie milisekund
idata BYTE timerS[MaxTimerS];                   //zliczanie sekund
data  WORD BlinkPeriod;
BYTE  pulse_bit;  
                                

// przygotowanie ukadu Timer0 -----------------------------------
void timer0_init(void) {
    TH0  = TH0RELOAD;              // zaadowanie czasu odliczania
    TL0  = TL0RELOAD;
    TMOD = TMOD | 0x01;                    // tryb 1 ukadu Timer0
    TR0  = 1;                                     //  Timer0 start
    ET0  = 1;                   // odblokowanie przerwa od Timer0 
}

// obsuga przerwania ukadu Timer0 ------------------------------
void T0_int(void)  interrupt 1 using 1 { 
  static WORD count=0,PulseCount=0; 
  BYTE i;
  TH0 = TH0RELOAD;                 // zaadowanie czasu odliczania                
  TL0 = TL0RELOAD;
 
  // modyfikacja licznikw odliczajcych milisekundy
  for(i=0;i<MaxTimerM;i++) if(timerM[i] > 0) timerM[i]--;

  // ustalenie czasu generacji przebiegu okresowego (blinking) 
  PulseCount++; 
  if((PulseCount > BlinkPeriod) && ( BlinkPeriod>0) ) {
                PulseCount=0;  pulse_bit=!pulse_bit;
  } 

  if (++count == SECOND)  {                   // odliczanie sekund
      count = 0;
        // modyfikacja licznikw odliczajcych sekundy
        for(i=0;i<MaxTimerS;i++) if(timerS[i] > 0) timerS[i]--;
     }
}

// zerowanie licznikw odliczajcych czasy -----------------------
void reset_timers(void) { 
  WORD i;        
  ET0  = 0;      
  for (i=0;i<MaxTimerM;i++)   timerM[i]= 0;  
  for (i=0;i<MaxTimerS;i++)   timerS[i]= 0;  
  ET0  = 1;      
}  

// ustawienie czasu odliczania milisekund -------------------------- 
void set_timerM(BYTE i, BYTE val) { 
  if(i < MaxTimerM) {
    ET0  = 0;   
    timerM[i] = val;
    ET0  = 1;   
  }
}

// ustawienie czasu odliczania milisekund -------------------------- 
void set_timerS(BYTE i, BYTE val) { 
  if(i < MaxTimerS) {
    ET0  = 0;   
    timerS[i] = val;
    ET0  = 1;   
  }
}

// czekanie zadan liczb sekund ------------------------------------
void delayS(BYTE val) {     
        set_timerS(0,val);
        while(timerS[0]) {}; 
}

// czekanie zadan liczb milisekund --------------------------------
void delayM(BYTE val) {       
        set_timerM(0,val);
        while(timerM[0]) {};           
}

//-------------------------------------------------------------------
void main(void) {  
    BYTE i=9,k=0x08;
    code digit[]= {     // definicje cyfr wywietlacza 8-segmentowego
                   0xc0,0xf9,0xa4,0xb0,              // cyfry 0,1,2,3
                   0x99,0x92,0x82,0xf8,              // cyfry 4,5,6,7
                   0x80,0x90,0x88,0x83,              // cyfry 8,9,A,b
                   0xC6,0xA1,0x86,0x8e               // cyfry c,d,E,F
    };
    timer0_init();                     // przygotowanie ukadu Timer0
    reset_timers();                      // zerowanie licznikw czasu
    BlinkPeriod=100;                                 //okres: 100msek
    
    EA  = 1;                                 // odblokowanie przerwa                  
    P3_7=0;   // przygotowanie wywietlacza LED (jedna cyfra aktywna)

    while(1) {
       LED= pulse_bit; //dioda LED bdzie pulsowaa z okresem 2*100ms
       if(timerS[1]==0) {  // co 2sek uaktywni si Buzzer na czas 1ms
            set_timerS(1,2); 
            set_timerM(1,10); 
            Buzzer=0;
       }
       if(timerM[1]==0) Buzzer=1;
       if(timerS[2]==0) {            // co 1sek pojawi si nowa cyfra
            set_timerS(2,1); 
            LEDS=digit[ i=(i+1) % 10 ]; // odliczanie w d 9...0
       }
       if(timerS[0]==0) {    // co 10sek w innym miejscu wywietlacza
            set_timerS(0,10);          //... pojawi si kolejna cyfra
            k= (k==0x80 ) ? 0x10 :k<<1;
            P3=~k;  // najstarsze 4-bity steruj pozycj wywietlania
      };
   }
}



