/*****************************************************************
timers_soft.c                   (uycie wielu ukadw odmierzania 
                                           ...sekund i milisekund)
Program pokazuje jak uywa jednoczenie wielu licznikw (timers)
odliczajcych czas. Liczniki odliczaj czas mierzony w 
milisekundach lub sekundach. Podstaw tego programu jest 
podprogram obsugi przerwania ukadu Timer0.
Rezultatu niezalenych licznikw to:

- co 1sek pojawi si cyfra od 0 do 9 na diodach LEDS i zczu SIO
- co 10sek pojawi si napis "co 10sek."
- co 2sek  brzczyk wyda dwik przez 20msek
*****************************************************************/
#include "51xD2.h"         // zbir definiujcy rejestry procesora 
#include "SIO_init.h"    // zbir makrodefinicji przygotowania SIO
                                                //... do dziaania
#include <stdio.h>         // definicje procedur wejcia - wyjcia
#define Fosc   24000000L               // czstotliwo oscylatora

#define  Buzzer     P1_3           // definicja adresu bitu Buzzer   
#define  LEDS       P2            // zesp diod wywietlacza LEDS

#define MaxTimerM   3            // liczba czasomierzy  milisekund  
#define MaxTimerS   3                 // liczba czasomierzy sekund
         
#define one_msek  (- Fosc/(12 * 1000) )           // 1 milisekunda       
#define TL0RELOAD    one_msek     
#define TH0RELOAD  ( one_msek )>>8        
#define SECOND     1000    // 1 sek wymaga 1000 przerwa od Timer0

idata BYTE timerM[MaxTimerM];              // zliczanie milisekund
idata BYTE timerS[MaxTimerS];                   //zliczanie sekund
data  WORD BlinkPeriod;
BYTE  pulse_bit;  
                                
// przygotowanie ukadu Timer0 -----------------------------------
void T0_init(void) {
    TH0  = TH0RELOAD;              // zaadowanie czasu odliczania
    TL0  = TL0RELOAD;
    TMOD = TMOD | 0x01;                    // tryb 1 ukadu Timer0
    TR0  = 1;                                     //  Timer0 start
    ET0  = 1;                   // odblokowanie przerwa od Timer0
}

// obsuga przerwania ukadu Timer0 ------------------------------
void T0_interrupt(void)  interrupt 1 using 1 { 
  static WORD count=0,PulseCount=0; 
  BYTE i;
  TH0 = TH0RELOAD;                 // zaadowanie czasu odliczania  
  TL0 = TL0RELOAD;
 
  // modyfikacja licznikw odliczajcych milisekundy
  for(i=0;i<MaxTimerM;i++) if(timerM[i] > 0) timerM[i]--;

  // ustalenie czasu generacji przebiegu okresowego (blinking) 
  PulseCount++;   if((PulseCount > BlinkPeriod) && ( BlinkPeriod>0) ) {
                PulseCount=0;  pulse_bit=!pulse_bit;
  } 

  if (++count == SECOND)  {                   // odliczanie sekund
      count = 0;
        // modyfikacja licznikw odliczajcych sekundy
        for(i=0;i<MaxTimerS;i++) if(timerS[i] > 0) timerS[i]--;
     }
}

// zerowanie licznikw odliczajcych czasy -----------------------
void reset_timers(void) { 
  WORD i;        
  ET0  = 0;      
  for (i=0;i<MaxTimerM;i++)   timerM[i]= 0;  
  for (i=0;i<MaxTimerS;i++)   timerS[i]= 0;  
  ET0  = 1;      
}  

// ustawienie czasu odliczania milisekund -------------------------
void set_timerM(BYTE i, BYTE val) { 
  if(i < MaxTimerM) {
    ET0  = 0;   
    timerM[i] = val;
    ET0  = 1;   
  }
}

// ustawienie czasu odliczania milisekund ------------------------ 
void set_timerS(BYTE i, BYTE val) { 
  if(i < MaxTimerS) {
    ET0  = 0;   
    timerS[i] = val;
    ET0  = 1;   
  }
}

// czekanie zadan liczb sekund ---------------------------------
void delayS(BYTE val) {     
        set_timerS(0,val);
        while(timerS[0]) {}; 
}

// czekanie zadan liczb milisekund -----------------------------
void delayM(BYTE val) {       
        set_timerM(0,val);
        while(timerM[0]) {};           
}

//----------------------------------------------------------------
void main(void) {  
    BYTE i=9;
 
    T0_init();                      // przygotowanie ukadu Timer0
    reset_timers();                   // zerowanie licznikw czasu
    SIO_initT1;                       // 9600BPS (8-N-1) dla 24MHz 
    EA  = 1;                              // odblokowanie przerwa                  

    while(1) {
       Buzzer= pulse_bit;         //ton ukadu Buzzer: okres 2*1ms
       if(timerS[1]==0) {  
            set_timerS(1,2);        // co 2sek uaktywni si Buzzer 
            set_timerM(1,20);                  // ... na czas 20ms
            BlinkPeriod=1;                        //okres: 2*1msek
       }
       if(timerM[1]==0)  BlinkPeriod=0;  
       if(timerS[2]==0) {          
            set_timerS(2,1);     // co 1sek pojawi si nowa cyfra
            i=(i+1) % 10;
            putchar(0x30 + i);         // odliczanie w gr 0...9
            LEDS=~i;
       }
       if(timerS[0]==0) {   
            set_timerS(0,10);                         // co 10sek        
            puts("\r\n co 10sek.  \n\r");  //... pojawi si napis
      };
   }
}

