/*****************************************************************
T_measure.c              (pomiar okresu T przebiegu prostoktnego)

Program mierzy okres przebiegu prostoktnego doczonego do 
wejcia T2EX (P1_1) ukadu Timer2. Ores mierzony jest 
z dokadnoci do 0.5usek.
Wynika to z czstotliwoci Fosc/12= 24MHz/12=2MHz  (T=0.5usek).
W momencie pojawienia si zbocza opadajcego na wejciu T2EX
pojawia si przerwanie i jednoczenie zapamitanie stanu ukadu
 Timer2. Okres mierzonego przebiegu to rnica czasw pomidzy 
dwoma przerwaniami od ukadu Timer2. Program wylicza take 
czstotliwo f przebiegu wykonujc dzielne cakowitoliczbowe.
Wyniki dziaania programu s wyprowadzane na zcze SIO: 9600bps 
(8-N-1). Do wyznaczenia prdkoci transmisji SIO uywany jest Time1.
Generator przebiegu prostoktnego, umoliwiajcy wygenerowanie 
wzorcowego przebiegu pomiarowego, zrealizowany jest za pomoc 
ukadu Timer0.

Uwaga: 
         Mierzony przebieg naley podczy do linii P1_1.
         Przebieg wzorcowy dostpny jest na linii P1_2 lub P3_1.
*****************************************************************/
#include "51xD2.h"         // zbir definiujcy rejestry procesora
#include "SIO_init.h"    // zbir makrodefinicji przygotowania SIO
                                                //... do dziaania

#include <stdio.h>                    // standardowe funkcje we/wy
#define  LED        P1_2                   // zewntrzna dioda LED
WORD current_val, old_val;

#define Fosc   24000000L               // czstotliwo oscylatora
#define one_msek  (- Fosc/(12 * 1000) )           // 1 milisekunda
#define TL0RELOAD    one_msek     
#define TH0RELOAD  ( one_msek )>>8        
#define SECOND 1000        // 1 sek wymaga 1000 przerwa od Timer0
#define CONST 1 //zmniejszenie czstotliwoci przebiegu wzorcowego


// obsuga przerwania ukadu Timer2 w trybie capture -------------
void T2_int(void)  interrupt 5 using 2 {
   TF2=0;
   if (EXF2==1) 	{    
      EXF2=0;
      old_val= current_val;
      current_val= ((WORD)RCAP2H<<8) | RCAP2L ;      // zapamitaj
                                   //... "zapany" (captured) czas
   }
}

//----------------------------------------------------------------
void main(void) {  
  WORD k, Period;
  TH0  = TH0RELOAD;                  
  TL0  = TL0RELOAD;
  TMOD |= 0x01;                         // tryb dla Timer0: mode1
  TR0  = 1;                                       // start Timer0
  ET0  = 1;                      // zgoda na przerwania od Timer0
  SIO_initT1;      // szybko transmisji zaley od ukadu Timer1
  T2CON=0x0d;        // EXEN2=1, TR2=1, CP/RL2=1 (external input,
                                  //... Timer2 run, compare mode)
  T2MOD=0;           // Timer2 output disable, down count disable

  ET2=1;                  // zgoda na przerwania od ukadu Timer2
  EA  = 1;                            // zezwolenie na przerwania
  printf("\r Pomiar okresu T przebiegu TTL ");

  while(1) { 
     Period=current_val - old_val;   // oblicz rnic w mikrosek
     printf("\r T= %u usek (f=%Lu x10-3 Hz) ",
             Period>>1, 2000000000L/(Period ) );
     for(k=0;k<0xffff;k++);                 // opnienie czasowe
  }                              
}

//podprogram przerwania -----------------------------------------
void T0_int(void)  interrupt 1 using 1 {   
 static BYTE count=0;
  TH0 = TH0RELOAD;                     // reload Timer0 registers
  TL0 = TL0RELOAD;
  if(++count==CONST) LED=!LED;              // mruganie diody LED
}

