/*****************************************************************
led_disp.c   (wywielanie 4-cyfr na wyswietlaczu LED przez Timer0)

Program wywietla zawarto zmiennej globalnej disp_val na 
wywietlaczu LED w postaci heksadecymalnej. Dynamiczne 
przeczanie niezbdna dla procesu wywietlania ukryto w 
podprogramie obsugi przerwania Timer0.
Aby przetestowa dziaanie programu w ptli gownej (nmain)
wywietla si wszystkie moliwe liczby 16-bitowe w postaci
heksadecymalnej.
*****************************************************************/

#include "51xD2.h"         // zbir definiujcy rejestry procesora 
#define  LEDS       P2            // zesp diod wywietlacza LEDS
#define  PLACE      P3          // pozycja do wywietlania na LEDS
           
#define TH0RELOAD  0xfc          //  0xfc18 = -1000 dla Fosc=12MHz
#define TL0RELOAD  0x18          //  ...    
#define SECOND     1000    // 1 sek wymaga 1000 przerwa od Timer0

idata BYTE timerS;
data  WORD disp_val=0x1234;                  // warto pocztkowa
BYTE  pulse_bit;  
                                
// przygotowanie ukadu Timer0 -----------------------------------
void timer0_init(void) {
    TH0  = TH0RELOAD;              // zaadowanie czasu odliczania
    TL0  = TL0RELOAD;
    TMOD = TMOD | 0x01;                    // tryb 1 ukadu Timer0
    TR0  = 1;                                     //  Timer0 start
    ET0  = 1;                   // odblokowanie przerwa od Timer0 
}

// obsuga przerwania ukadu Timer0 ---------------------------------
void T0_int(void)  interrupt 1 using 1 { 
  static WORD count=0;
  static BYTE i,k=0x08;     // k okrela pozycj cyry do wywietlania
  code digit[]= {       // definicje cyfr wywietlacza 8-segmentowego
                   0xc0,0xf9,0xa4,0xb0,              // cyfry 0,1,2,3
                   0x99,0x92,0x82,0xf8,              // cyfry 4,5,6,7
                   0x80,0x90,0x88,0x83,              // cyfry 8,9,A,b
                   0xC6,0xA1,0x86,0x8e               // cyfry c,d,E,F
  }; 
  TH0 = TH0RELOAD;         // zaadowanie czasu odliczania              
  TL0 = TL0RELOAD;
  // wywietlanie zawartoci disp_val na wywietlaczu LEDS
  PLACE=0xff;
  LEDS=digit[ ( disp_val>>(4*i) ) & 0x0f ];  // co wywietli na LEDS
  k= (k==0x80 ) ? 0x10 :k<<1; 
  PLACE=~k;                               // gdzie wywietli na LEDS
  i= (i+1) & 0x03;
  if (++count == SECOND)  {                      // odliczanie sekund
        count = 0;
        // modyfikacja licznika odliczajcego sekundy
        if(timerS > 0) timerS--;
     }
}

// czekanie zadan liczb sekund ------------------------------------
void delayS(BYTE val) {      
     ET0  = 0;   
     timerS = val;
     ET0  = 1;   
     while(timerS) {}; 
}

//-------------------------------------------------------------------
void main(void) {  
     timer0_init();                    // przygotowanie ukadu Timer0   
     EA  = 1;                                // odblokowanie przerwa    

     while(1) {  
         disp_val++;            // wylicz now liczb do wywietlania
         delayS(1);                                    // czekaj 1sek
    }
}



