/***********************************************************************
LCD4bit.c                     (biblioteka LCD dla 4-bitowego interfejsu)

Biblioteka obsugi wywietlacza LCD podczonego do
portu poprzez 4-linie danych.
Wywietlacz LCD podczony jest do portu P0:
   LCD          uP
   -------------------
   DB4        = P0.0
   DB5        = P0.1
   DB6        = P0.2
   DB7        = P0.3

   D/C~ (RS)  = P0.4        Data/Control~
   R/W~       = P0.5        Read/Write~
   E          = P0.6        Enable
************************************************************************/
#include "51xD2.h"                // zbir definiujcy rejestry procesora 
#include <lcd.h>                                         // definicje LCD 

#define DATA_LCD P0                 // wywietlacz podczony do portu P0  
#define DC_LCD   P0_4                        // linia D/C~ (Data/Control) 
#define RW_LCD   P0_5                          // linia R/W~ (Read/Write) 
#define E_LCD    P0_6                                 // linia E (Enable) 
#define INTERFACE LCD_4BIT

// ------------------------  sprawdzenie flagi zajtoci wywietlacza LCD 
BYTE check_busy (void) {
    BYTE bf;
    DC_LCD = 0;          // linia D/C~=0 : odczyt sowa Control (statusu)  
    RW_LCD = 1;                                  // linia R/W~=1 : odczyt  
    E_LCD = 1; 
    bf = DATA_LCD & 0x08;     // najstarszy bit w czwrce to zajto LCD 
    E_LCD = 0; 
    E_LCD = 1;                 // linia Enable E=0: odczyt z wywietlacza  
    E_LCD = 0;                         // linia Enable E=1:koniec odczytu 
    return (bf);
}

//------------------------------------- zapis 4-bitw do wywietlacza LCD 
void write_nibble ( BYTE nibble) {
  BYTE x;
  E_LCD = 0;                           // Enable=1 wywietlacz nieaktywny 
  RW_LCD = 0;                                      // linia zapisu R/W~=0  
  x= DATA_LCD & 0xF0;
  nibble &= 0x0F;                   // tylko 4-modsze bity bd zapisane  
  DATA_LCD = nibble | x;    //dane s przekazywane na liniach P0_3...P0_0
  E_LCD = 1;                   // linia Enable E=0: zapis do wywietlacza  
  E_LCD = 0;                           // linia Enable E=1: koniec zapisu  
  DATA_LCD = x | 0x0F;               // powrt linii danych do stanu 0x0F
}

//------------------------------------- odczyt 4-bitw z wywietlacza LCD 
BYTE read_nibble (void) {
BYTE nibble;
E_LCD = 0;                                                    // Enable=0 
RW_LCD = 1;                                             // odczyt: R/W~=1  
E_LCD = 1;                     // linia Enable E=1: odczyt z wywietlacza  
nibble= DATA_LCD & 0x0f;                                 // odczyt danych  
E_LCD = 0;                            // linia Enable E=0: koniec odczytu  
return(nibble);
}

//------------------- zapis sowa sterujcego Control do wywietlacza LCD 
void LCD_ctrlWR ( BYTE cmde) {
    while (check_busy() );          // sprawdzenie zajtoci wywietlacza  
    DC_LCD = 0;                            // wybierz rejestr Control LCD  
    write_nibble(cmde >> 4);      // zapis starszej czwrki bajtu Control 
    write_nibble(cmde);           // zapis modszej czwrki bajtu Control 
}

//------------------- odczyt sowa sterujcego Control z wywietlacza LCD 
BYTE LCD_ctrlRD ( void) {
BYTE byte;
while (check_busy() );              // sprawdzenie zajtoci wywietlacza  
DC_LCD = 0;                                // wybierz rejestr Control LCD  
byte =read_nibble() <<4;         // odczyt starszej czwrki bajtu Control 
byte =byte | read_nibble() ;     // odczyt modszej czwrki bajtu Control 
return( byte);
}

//--------------------------------- zapis danych Data do wywietlacza LCD 
void LCD_dataWR ( BYTE byte) {
   while (check_busy() );           // sprawdzenie zajtoci wywietlacza  
   DC_LCD = 1;                         // wybierz rejestr danych Data LCD  
   write_nibble(byte >> 4);          // zapis starszej czwrki bajtu Data 
   write_nibble(byte);               // zapis modszej czwrki bajtu Data 
}

//--------------------------------- odczyt danych Data z wywietlacza LCD 
 BYTE LCD_dataRD ( void) {
BYTE byte;
while (check_busy() );              // sprawdzenie zajtoci wywietlacza  
DC_LCD = 1;                            // wybierz rejestr danych Data LCD  
byte =read_nibble() <<4;            // odczyt starszej czwrki bajtu Data 
byte =byte | read_nibble() ;        // odczyt modszej czwrki bajtu Data 
return( byte);
}

#include "lcd_comm.c"
