/***********************************************************************
clock.c                                           ( zegar elektroniczny) 

Program demonstruje dziaanie elektronicznego zegara. Jest on
modyfikacj programu rdowego prezentowanego przez firm
Keil. Komunikaty zegara wyprowadzane s na wywietlacz LCD lub 
zcze SIO transmisji szeregowej z uyciem funkcji z biblioteki stdio.h.
Podczas obsugi wywietlacza dodano obsug znaku nowej linii \n w 
zbiorze LCD_comm.c. Poprawiono take zbir KEYB.c aby dostosowa 
klawiatur telefoniczno do programu obsugi zegara.
***********************************************************************/
//#define LCD           // usu ca linie dla transmisji szeregowej SIO
#include "51xD2.h" 
#include <stdio.h>
#include "sio_init.h"
#include "lcd.h"
#define FREQ   24000000L     // czstotliwo oscylatora kwarcowego w Hz

#define PERIOD  200        // liczba autoprzeadowania dla ukadu Timer0
#define SECOND  FREQ/(12*PERIOD) // czas jednej sekundy: liczba przerwa

struct time  {
    BYTE   hour;                                              // godziny
    BYTE   min;                                                // minuty
    BYTE   sec;                                               // sekundy
} clk_time;                                             // aktualny czas

//----------------------------------------------------------------------
// Podprogram przerwania od ukadu Timer0 odlicza godziny, minuty, sek.
//----------------------------------------------------------------------
void T0_int (void) interrupt 1 using 1 {
  WORD  count;                    // licznik wywoania przerwania Timer0
  if (++count == SECOND)  {                  // zwiksz licznik przerwa
    count = 0;                      // Zmiana czasu nastpuje co sekund
    if (++clk_time.sec == 60)  {               // zwiksz licznik sekund
      clk_time.sec = 0;                      // ustaw licznik minut na 0
      if (++clk_time.min == 60)  {              // zwiksz licznik minut
        clk_time.min = 0;                   // ustaw licznik godzin na 0
        if (++clk_time.hour == 25)  {          // zwiksz licznik godzin
          clk_time.hour = 1;
        }
      }
    }
  }
}

// Wywietlanie repertuaru moliwoci zegara.---------------------------
// Podprogram zwraca znak wybranej funkcji zegara.
//----------------------------------------------------------------------
char menu(void) {
  printf ("\nClock");                               // wywietl nagwek
  printf ("\n0-disp *-set");               // wywietl moliwoci zegara
  return (_getkey());                   // pobierz znak - bez echa znaku
}

// Uaktualnij czas dnia ------------------------------------------------
void update (struct time *dtime) {
  printf("\r%02bd:%02bd:%02bd   ", dtime->hour,dtime->min,dtime->sec );
}

// Wywietlanie czasu dnia co jedna sekund.----------------------------
// Wcinicie dowolnego przycisku powoduje zakoczenie tej procedury.
//----------------------------------------------------------------------
void display(void) {
  unsigned char last = 0xFF;             // ostatnio wywietlona sekunda

  printf("\nTIME/#-menu\n");

  while (1)  {

       #ifdef LCD
         P1 &=0x0f;
         if ( P1 !=0x0f  )  { P1=0xff; break; }           //dowolny znak
       #else
         if ( RI   ) {RI=0; break;}                      // dowolny znak
       #endif

       if (last != clk_time.sec)  {   // sprawdzenie czy uaktualni czas
       update(&clk_time);                          // wywietl nowy czas
       last = clk_time.sec;
     }
  }
}


// Ustawianie czasu-----------------------------------------------------
// Wcinicie przycisku CR powoduje zakoczenie tej procedury
//----------------------------------------------------------------------
set_time (struct time *dtime ) {
  struct time setuptime;
  setuptime.hour = dtime->hour;                            // kopiowanie
  setuptime.min  = dtime->min;
  setuptime.sec  = dtime->sec;

  printf ("\n123:T/#-Strt/*-Quit\n");         // Nacinij 123 dla zmiany
                                             //... godzin, minut, sekund.
                                             // Nacinij # aby uruchomi
                                                     //... zegar - start.
                                             //  Nacini * aby zakoczy
                                               //... wprowadzanie - Quit.
  update(&setuptime);                            // uaktualnij czas dnia

  while (1)  {
    switch (_getkey())  {                     // pobierz znak - bez echa

      case '#':                           // ustaw aktualny czas gdy "#"
        dtime->hour = setuptime.hour;
        dtime->min  = setuptime.min;
        dtime->sec  = setuptime.sec;
        return;

      case '1':                       // zmie godziny gdy wcinito "1"
        if (++setuptime.hour == 25)  setuptime.hour = 1;
        break;

      case '2':                       // zmie minuty gdy wcinito  "2"
        if (++setuptime.min == 60)  setuptime.min = 0;
        break;

      case '3':                       // zmie sekundy gdy wcinito "3"
        if (++setuptime.sec == 60)  setuptime.sec = 0;
        break;

      default:
        continue;
    }
    update(&setuptime);                            // skoryguj czas dnia
  }
}

//----------------------------------------------------------------------
void main(void) {
  clk_time.hour = 12;                         // inicjalizacja zmiennych

  #ifdef LCD
    LCD_init;                                       // inicjalizacja LCD
  #else
    SIO_initT1;                                     // inicjalizacja SIO
  #endif


  TL0 =TH0 = -PERIOD;       // ustawienie okresu dziaanie ukadu Timer0
  TMOD = TMOD | 0x02;            // tryb Timer0: MODE2 autoprzeadowanie
  TR0 = 1;                                       // uruchom ukad Timer0
  ET0 = 1;                  // zezwolenie na przerwania od ukadu Timer0
  EA  = 1;                             // zezwolenie na obsug przerwa


  while (1)  {                                        // ptla bez koca

    switch (menu())  {                    // repertuar moliwoci zegara
      case '0': display();                         // wywietl czas dnia
                break;
      case '*': set_time(&clk_time);                    // zmie czas dnia
                break;
      default:  break;
    }
  }
}

