/*********************************************************************
circ_intr.c            (biblioteka moduw obsugi SIO z przerwaniami)
                           ... i buforami koowymi

Najwaniejsze procedury tej biblioteki to _getkey() i putchar().
Bdy nadawania lub odbioru powstaj gdy TXByteCount > TXBUFSIZE lub
RXByteCount > RXBUFSIZE. 
Wywoanie   if(RXByteCount > 0) _getkey()
zabezpiecza przed utkniciem w ptli czekania na bajt. 
Dziaanie bufora koowego TX mona wyczy i zastpi go 
bezporednim wysyaniem znaku z czekaniem na zakoczenie transmisji
(kompilacja warunkowa TXCIRC).
*********************************************************************/
#define TXCIRC              // jeli bufor TX na by buforem koowym

#include "SIO_init.h"
#define RXBUFSIZE 16               // rozmiar bufora odebranych bajtw
xdata BYTE RXBuf[RXBUFSIZE];                // bufor odebranych bajtw
BYTE RXIndexRD, RXIndexWR;         // indeksy bufora odebranych bajtw
BYTE RXByteCount;                         // licznik odebranych bajtw
#define TXBUFSIZE 16               // rozmiar bufora nadawanych bajtw
xdata BYTE TXBuf[TXBUFSIZE];                // bufor nadawanych bajtw
BYTE TXIndexRD, TXIndexWR;         // indeksy bufora nadawanych bajtw
BYTE TXByteCount;                         // licznik bajtw do nadania
BYTE TXBufEmpty;    // znacznik gdy bufor nadawanych bajtw jest pusty

#define DIR485 P3_2          // bit kierunku transmisji dla ukadu 485

// przygotownie transmisji SIO ---------------------------------------
void sio_init (void) {
  RXIndexWR = RXIndexRD = RXByteCount = 0;
  TXIndexWR = TXIndexRD = TXByteCount = 0;
  TXBufEmpty = 1;                              // bufor TX jest pusty
  SIO_initT2;      // ustawienie prdkoci transmisji: T1, T2 lub BDR
  ES=1;                            // zezwolenie na przerwania od SIO
  DIR485=0;                                   // kierunek 485: odbir
}
// obsuga przerwania SIO -------------------------------------------
void SCON_int (void) interrupt 4 {  // Serial Port 0
 // EA=1;

  #ifdef TXCIRC
  if (TI) {//przerwanie gdy ukad nadawczy SIO jest pusty po wysaniu
    TI=0;
    DIR485 = 0;                               // kierunek 485: odbir
    if (TXByteCount)  {
      DIR485=1;                            // kierunek 485: nadawanie
      SBUF = TXBuf[TXIndexRD++];          // kolejny bajt do wysania
      TXIndexRD &= TXBUFSIZE-1;
      TXByteCount--;
    }
    else   TXBufEmpty = 1;          // gdy nie ma ju nic do wysania
  }

  #else
  if (TI) {              // bez obsugi bufora koowego tarnsmisji TX
    TI=0;  //przerwanie gdy ukad nadawczy SIO jest pusty po wysaniu;          
    DIR485 = 0;                               // kierunek 485: odbir     
  } 

  #endif
  if (RI) {                            //przerwanie gdy odebrano bajt
     RI=0;
     RXBuf[RXIndexWR++] = SBUF;         // przelij bajt do bufora RX
     RXIndexWR &= RXBUFSIZE-1;                  // operacja cykliczna
     RXByteCount++;                      // licznik odebranych bajtw
  }

}

// podprogram putchar zgodny z stdio.h ------------------------------
// Zapis bajtu do bufora TX. Gdy bufor TX jest pusty wysanie bajtu
// bezporednio. Reszta bajtw z bufora TX wysya podprogram 
// przerwania SIO w sposb automatyczny.
char putchar (char Byte) {

   #ifdef TXCIRC
   while((TXByteCount>=(RXBUFSIZE-1) ) && !TXBufEmpty ) {}; // czekaj
   ES = 0;                                 // blokowanie przerwa SIO
   TXBuf[TXIndexWR++] = Byte;             // zapisz bajt do bufora TX
   TXIndexWR &= TXBUFSIZE-1;               // obsuga bufora koowego
   TXByteCount++;           // zwikszenie licznika znakw do nadania
   ES = 1;                               // odblokowanie przerwa SIO
   if (TXBufEmpty && TXByteCount>0 ) { // bufor TX zawiera 1-szy bajt
        ES = 0;                            // blokowanie przerwa SIO
        TXBufEmpty = 0;                // bufor zawiera pierwszy bajt
        DIR485=1;                          // kierunek 485: nadawanie
        SBUF = TXBuf[TXIndexRD++];                     // wylij bajt
        TXIndexRD &= TXBUFSIZE-1;          // obsuga bufora koowego
        TXByteCount--;     // zmniejszenie licznika znakw do nadania
        ES = 1;                          // odblokowanie przerwa SIO
   }

   #else                           // obsuga SIO bez bufora koowego
   do {} while (DIR485==1 );     // czekanie na zakoczenie nadawanie
   DIR485=1;                               // kierunek 485: nadawanie
   SBUF = Byte;                              // wysanie nowego bajtu
   #endif

   return (Byte);               // tak zdefiniowano putchar w stdio.h
}

// podprogram _getkey zgodny z stdio.h ------------------------------
// podprogram pobiera kolejny bajt z bufora RX
char _getkey (void) {
  char Byte;
  while (RXByteCount==0) {};   // czekanie na bajt gdy bufor RX pusty
  ES = 0;                                  // blokowanie przerwa SIO
  Byte = RXBuf[RXIndexRD++];            // pobranie bajtu z bufora RX
  RXIndexRD &= RXBUFSIZE-1;            // operacja na buforze koowym
  RXByteCount--;             // poprawienie licznika znakw pobranych
  ES = 1;                                // odblokowanie przerwa SIO
  return (Byte);                                          
}

// kasowanie bufora RX ----------------------------------------------
void flush_RX(void) {
    ES = 0;                                // blokowanie przerwa SIO
    RXByteCount = 0; RXIndexRD = 0;            // indeksy na pocztek
    ES = 1;                              // odblokowanie przerwa SIO
}

// liczba znakw w buforze RXCount ----------------------------------
BYTE  RX_Count(void) {
   return(RXByteCount);
}


