#include "SIO_init.h"
#define RXBUFSIZE 16                                      // receive buffer size
xdata BYTE RXBuffer[RXBUFSIZE];         // receive buffer
BYTE RXReadIndex, RXWriteIndex; // receive buffer indexes
char RXCharCount;                           // count of received bytes
#define TXBUFSIZE 16                                       // transmit buffer size
 xdata BYTE TXBuffer[TXBUFSIZE];         // transmit buffer
 BYTE TXReadIndex, TXWriteIndex; // transmit buffer indexes
 BYTE TXCharCount;                 // not yet transmitted bytes
#define BUFFER_EMPTY 1                                       // 1: nothing to send
 BYTE bTXBufferEmpty;                // flag for synchronization


void UART_INIT (void) {
  SCON = 0x52;          // Mode 1, REN0=1
  TMOD=0x21;
  TL1=TH1=-13;     //f=12MHz BAUD=4800bps
  PCON |= 0x80;
  TR1=1;
  IE = IE | 0x10;      // Enable Serial 0
}


//--------------------------------------------------------- Init the RS232 driver system
void sio_init (void) {
  RXWriteIndex = RXReadIndex = RXCharCount = 0;
  TXWriteIndex = TXReadIndex = TXCharCount = 0;
  bTXBufferEmpty = BUFFER_EMPTY;                               // reset empty flag
  UART_INIT();                                                   // setup interface:
}


// ----------------------------------Stores one char in TX buffer. If it's the first one,
//                         send it immediately. Rest is sent by TXInterrupt automatically
char putchar (char cByte) {
   while((TXCharCount>=(RXBUFSIZE-1) ) && ! bTXBufferEmpty );
   TXBuffer[TXWriteIndex++] = cByte;             // load byte to buffer and inc index
   TXWriteIndex &= TXBUFSIZE-1;                    // adjust index to borders of buffer
   ES = 0;                                         // disable serial 0 int                                 // disable transmit interrupt (in IE2)
   TXCharCount++;                                  // new char, inc count
   ES = 1;                                         // enable serial 0 int                                   // enable interrupt (in IE2)
   if (bTXBufferEmpty && TXCharCount)   {          // buffer had been empty   
        SBUF = TXBuffer[TXReadIndex];
        bTXBufferEmpty = !BUFFER_EMPTY;              // reset empty flag
        SBUF = TXBuffer[TXReadIndex++];             // load tx register, inc index
        TXReadIndex &= TXBUFSIZE-1;
        TXCharCount--;
   }
   return cByte;
}


void SCON_int (void) interrupt 4  using 1                // Serial Port 0
{ 
  EA=1;
  if (TI) { //--- Transmiter empty

    TI=0;
    if (TXCharCount)  {
      SBUF = TXBuffer[TXReadIndex];            // load tx register, inc index
      TXReadIndex++;
      TXReadIndex &= TXBUFSIZE-1;
      TXCharCount--;
    }
    else   bTXBufferEmpty = BUFFER_EMPTY;                   // set empty flag
  }

  if (RI) { //---- Receiver full

    RI=0;
    RXBuffer[RXWriteIndex++] = SBUF;              // store received byte and
    // inc receive index
    RXWriteIndex &= RXBUFSIZE-1;                      // reset index
    RXCharCount++;                                    // received, inc count
  }

}

// How many chars are stored in RX buffer ?
// if main routine wants to read chars, it has
// to check first if RS232RXBufferCount() returns !=0

BYTE RXBufferCount (void) {
  return (RXCharCount);
}

// ---------------------Get one char from RX buffer. Multiple calls will return all chars.
char _getkey (void) 
{
  char Byte;
  while (RXCharCount==0);
    // char still available
     Byte = RXBuffer[RXReadIndex++];                 // get byte from buffer
     RXReadIndex &= RXBUFSIZE-1;                       // adjust index
     ES = 0;       // disable serial 0 int                                   // disable rx interrupt (IE2)
     RXCharCount--;                                    // one char read, dec count
     ES = 1;       // enable serial 0 int                                      // done, enable int (IE2)
     return (Byte);  // if there is no new char                                         
}


void flush_rx(void)
// flush SIO dev RX buffer
{
    ES = 0;  // disable serial 0 int
    RXCharCount = 0; RXReadIndex = 0;
    ES = 1;  // enable serial 0 int
}

#define LINE_LENGTH 20		// Change if you need
#define BACKSPACE 0x08       // ASCII <DEL>

                                                                        
// -----------Pobranie jednej linii tekstu z moliwoci edycji znakw                      
void getline (char *line, BYTE n)  {
  char cnt = 0;
  char c;

  do  {
    if ((c = _getkey()) == '\r')  c = '\n';       // pobranie znaku 
    if (c == BACKSPACE  )  {             // obsuga znakw Bakspace(0x08) 
      if (cnt != 0)  {
        cnt--;                            // zmniejszenie licznika znakw 
        line--;                 // zmniejszenie wskanika biecego znaku 
        putchar (BACKSPACE);             // wyprowadzenie znaku Backspace 
        putchar (' ');
        putchar (BACKSPACE);
      }
    }
    else  {
      putchar (*line = c);    // echo znaku i skadowanie znaku w buforze 
      line++;                     // zwikszenie wskanika znakw w linii 
      cnt++;                               // zwikszenie licznika znakw 
    }
  }  while (cnt < n - 1  &&  c != '\n');     // sprawdzenie limitu znakw 
  *line = 0;                  // znak 0 (NULL) jest ogranicznikiem znakw 
}


