/*********************************************************************
AT25040.c                      (biblioteka moduw pamici SPI EEPROM)
Biblioteka moduw obsugi pamici SPI typu AT25010, AT25020 i
At25040. W zalenoci od producenta pamici moliwe jest programowanie
pojedynczych bajtw i blokw pamici.
*********************************************************************/
#include "51xD2.h"
#include "spi_lib.h"
#include "AT25040.h"

// -------------------------------------------- odczyt statusu AT25040
unsigned char AT25040_read_status(void )  {
     unsigned char spi_data;
     AT25040_CS = 0;                     // chip select - wybr ukadu
     SPI_byte(AT25040_CMD_RDSR);
     spi_data=SPI_byte(0x55);                       // dowolna warto
     AT25040_CS = 1; 
     return (spi_data);
}

// -----------------------------  prawdzenie gotowoci pamici AT25040
bit AT25040_ready(void)  {
     return ( AT25040_read_status() & AT25040_MSK_RDY) ;
} 

// -------------------------- ustawienie gody na zapis pamici AT25040
void AT25040_set_write_enable(void )  {
     AT25040_WAIT;
     AT25040_CS = 0;                      
     SPI_byte(AT25040_CMD_WREN);
     AT25040_CS = 1; 
}

// -------------------------- kasowanie zgody na zapis pamici AT25040
void AT25040_clear_write_enable(void ) {
     AT25040_WAIT;
     AT25040_CS = 0;
     SPI_byte(AT25040_CMD_WRDI);
     AT25040_CS = 1; 
}

// ------------------------------------ zapis bajtu do pamici AT25040
void AT25040_WR_byte(BYTE adr, BYTE to_write )  {
     AT25040_WAIT;
    AT25040_WR_ENABLE;
     AT25040_CS = 0;
    SPI_byte(AT25040_CMD_WRITE);
    SPI_byte(adr);
    SPI_byte(to_write);
     AT25040_CS = 1; 
}

// ------------------------------------ odczyt bajtu z pamici AT25040
BYTE AT25040_RD_byte(BYTE adr )  {
     BYTE spi_data;
    AT25040_WAIT;
     AT25040_CS = 0;
    SPI_byte(AT25040_CMD_READ);
     SPI_byte(adr);
     spi_data=SPI_byte(0xff);                       // dowolna warto
     AT25040_CS = 1; 
     return (spi_data);
}

// ----------------------------------- zapis danych do pamici AT25040
void AT25040_WR_data( BYTE adr, BYTE  *ptr_data, BYTE nb_data) {
    adr     &= ~(AT25040_MAX_PAGE -1);       //zapis na strony pamici
     nb_data &=   AT25040_MAX_PAGE -1 ;
    AT25040_WAIT;
    AT25040_WR_ENABLE;
     AT25040_CS = 0;
     SPI_byte(AT25040_CMD_WRITE);
     SPI_byte(adr);     
    SPI_data( ptr_data,nb_data);
     AT25040_CS = 1; 
}

// ------------------------------odczyt cigu danych z pamici AT25040
void AT25040_RD_data( BYTE adr, BYTE  *ptr_data, BYTE nb_data) {
     AT25040_WAIT;
    AT25040_CS = 0;
     SPI_byte(AT25040_CMD_READ);
     SPI_byte(adr);
     SPI_data( ptr_data, nb_data); 
     AT25040_CS = 1; 
}


