/*********************************************************************
SPI_eeprom.c                 (zapis i odczyt do pamici EEPROM  25040)

Program zapisuje i odczytuje dane do pamici EEPROM 25010,25020 
i 25040 z interfejsem SPI. Procesor steruje ukadowym
interfejsem SPI wystpujcym w procesorach Atmel lub obsuguje 
interfejs SPI metod programow (softrware'owo). Aby wybra sposb 
obsugi interfejsu SPI wystarczy wymieni modu programu
       spi_lib_hard.c - obsuga ukadowa interfejsu SPI
       spi_lib_soft.c - obsuga programowa (software'owa) SPI .
Na projekt skadaj sie:
   SPI_eeprom.c - modu gwny obsugi pamici EEPROM SPI
   SPI_lib_soft.c - biblioteka SPI
   at25040.c - biblioteka podprogramw obsugi pamici EEPROM.
Zapis i odczyt do pamici odbywa si dwoma metodami. Pierwsza 
metoda to zapis i odczyt bajt po bajcie osobno. Druga metoda to zapis
i odczyt blokami (stronami). Moliwo zapis i odczytu blokami zaley 
od typu pamici. Firma Xicor zezwala na zapis blokami typu WORD po dwa 
bajty. Firma Atmel zezwala na zapis stronami po 8-bajtw.
Wyniki dziaania programu wyprowadzane s na zcze SIO: 9600bps 8-N-1.
**********************************************************************/

#include "51xD2.h"                     // rejestry SFR procesora '51xD2
#include "SIO_init.h"               // przygotowanie SIO: 9600bps 8-N-1
#include "AT25040.h"            // obsuga pamici 25010, 25020 i 25040
#include "spi_lib.h"     // obsuga SPI interfejsu w procesorach '51xD2
#include <stdio.h>     


#define SS                     P1_1     // linia SS/P1_1 interfejsu SPI
#define BUFFER_SIZE             255      
extern  BYTE SPI_error;                // licznik bdw transmisji SPI

// --------------------------------------------------------------------
void main(void) {

        BYTE adr;
        BYTE xdata buf[BUFFER_SIZE];       // bufor danych wysyanych i
                                                // ...odbieranych z SPI
        SIO_initBDRf;               // przygotowanie SIO: 9600bps 8-N-1


        printf("\n\nBiblioteka SPI (obsuga programowa - soft)\n");

#ifdef SPI_hard
        SS=1;                      // linia select dla trybu MASTER SPI

        //   przygotwanie SPI :  MASTER, clock=Fosc/128, SS disable ...
        SPCON  = SPI_RATIO_128 | MSK_SPI_MSTR | MSK_SPI_SSDIS ;
        SPCON |= MSK_SPI_SPEN;                            // SPI enable
        SPSTA =0x80;                       // gotowo do nadawania SPI
#endif        
        AT25040_clear_write_enable();     // blokowanie zapisu do 25040
        printf("\nStatus: %02bx",AT25040_read_status()); 
        AT25040_set_write_enable();     // zezwolenie na zapis do 25040
        printf("\nStatus: %02bx",AT25040_read_status());

        printf("\nodczyt [0x55]: %02bx", AT25040_RD_byte(0x55));
        printf("\nzapis  [0x55} = 0x55");

        AT25040_WR_byte(0x55,0x55);
        printf("\nodczyt [0x55]: %02bx", AT25040_RD_byte(0x55));

        printf("\nzapis  [0x55] = 0xAA");
        AT25040_WR_byte(0x55,0xAA);
        printf("\nodczyt [0x55]: %02bx", AT25040_RD_byte(0x55));

        printf("\n ZAPIS 0..255");
        
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) 
                                     AT25040_WR_byte(adr,adr);
        printf("\n ODCZYT 0..255\n");  
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) 
                          printf("%02bx ", AT25040_RD_byte(adr));
        
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) buf[adr]=0;  
  
        printf("\n Cigy odczyt 0..255 \n");      
        AT25040_RD_data( 0, &buf,  BUFFER_SIZE);
        for ( adr=0;adr < BUFFER_SIZE;adr++ ) printf("%02bx ",buf[adr]);
 
        for ( adr=0; adr < BUFFER_SIZE; adr++ ) 
                                            buf[adr]=~adr; // nowy bufor

        printf("\n Cigy zapis 0..1 \n");        
        AT25040_WR_data( 0, &buf, 2);      // zapis sowa word (2 bajty)
        printf("\n Cigy odczyt 0..1 \n");       
        AT25040_RD_data( 0, &buf, BUFFER_SIZE);// odczyt sowa (2 bajty)
        for ( adr=0; adr < 2; adr++ )   printf("%02bx ",buf[adr]);

    printf("\nLiczba bdw: %bu", SPI_error);
    while(1);
}

